#!/usr/bin/python
'''
Created on Jun 6, 2011
@author: rchen
Create a bunch of extrinsicObjects that the registry uses. No objects.
'''

import sys, getopt, re
import time
from pds4regURL import pds4regURL

lidBase = "stressTesting"
counterBase = 0
urlBase = "http://localhost:8080/registry/extrinsics"
numProducts = 3
numIReports = 0
doDelete = True
doRegister = True
doView = True
viewLite = False
# keep the following 3 vars synched
templateLID  = "__lid__.T__counter__"	# no longer used. View used to need this
templateGUID = "__lid__.T__counter__.v1.0"	# needed to delete
templateRegister = """<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<extrinsicObject xmlns="http://registry.pds.nasa.gov"
    lid="__lid__.T__counter__"
    guid="__lid__.T__counter__.v1.0"
    versionId="1.0"
    name="Stress The Registry "
    objectType="Product"
    description="Stolen from http://pdscm/2010/registry/registry-service/operate/index.html with attribute lid above changed for uniqueness" >
    <slot name="first-name">
        <value>John</value>
    </slot>
    <slot name="last-name">
        <value>Doe</value>
    </slot>
    <slot name="phone">
        <value>(818)777-7777</value>
        <value>(818)888-8888</value>
    </slot>
    <slot name="cannotPossibleBeAnExistingSlot">
        <value>cannot possibly be an existing value</value>
    </slot>
</extrinsicObject>"""

def getStats(list):
	"given a list of numbers, return average, median, standard dev as a tuple"
	total = sum(list)	# length might reach 1000000, so do this once
	length = len(list)
	avg = total * 1.0 / length
	sList = sorted(list)
	med = sList[(length+1)/2 -1]	# works for list with odd length
	if length % 2 == 0:
		med = (sList[length/2-1] + sList[length/2]) / 2.0
	sumDiffSquared = 0
	for n in range(length):
		sumDiffSquared += (list[n]-avg)**2
	sd = (sumDiffSquared / length)**0.5
	return length, avg, med, sd, total

# accesses global vars lidBase, counterBase, numProducts, numIReports
def doWork(func, template, funcName):
	"register, view, delete do essentially the same thing. this thing"
	ir = 1
	intermediateTimes = []
	timeGood = []
	timeBad = []
	t2 = template.replace("__lid__", lidBase)
	for i in range(numProducts):		# legal to access numProducts?
		t3 = t2.replace("__counter__", "%06d" % (i + counterBase))
		time0 = time.time()
		try: ret = func(t3)
		except: timeBad.append(time.time() - time0)
		else:
			timeGood.append(time.time() - time0)
			if viewLite:
				m = re.search("lid\s*=\s*\"(.*?)\"", ret)
				if m: ret = "..." + m.group(1) + "..."
				else: ret = ret[0:50]
			print ret
		if (numIReports > 0):
			intermediateTimes.append(time.time() - time0)
			if (i+1 >= ir * numProducts / numIReports):
				stats = getStats(intermediateTimes)
				print "%-6s %d/%d. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % (funcName, ir, numIReports, stats[1], stats[2], stats[3], stats[4])
				ir += 1
				intermediateTimes = []
	# print() needs a tuple? How to create one from a var and return values?
	if len(timeGood) > 0:
		stats = getStats(timeGood)
		print "%-6s %6d good. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % (funcName, stats[0], stats[1], stats[2], stats[3], stats[4])
	if len(timeBad) > 0:
		stats = getStats(timeBad)
		print "%-6s %6d fail. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % (funcName, stats[0], stats[1], stats[2], stats[3], stats[4])

def usage():
	print "usage: ", sys.argv[0], "[-hDRVvn:i:c:l:u:]"
	print "  -h help"
	print "  -D do NOT delete from registry (happens after viewing)"
	print "  -R do NOT register"
	print "  -V do NOT view (after registering)"
	print "  -v view but return shortened output"
	print "  -n<number> products to register. Default: %d" % numProducts
        print "  -i<number> intermediate reports. Default: %d" % numIReports
	print "  -l<lid> base of logicalID.   Default: " + lidBase
	print "  -c<num> base of LID counter. Default: %d" % counterBase
	print "  -u<url> base of registry.    Default: " + urlBase
	sys.exit()

try:
	opts, args = getopt.getopt(sys.argv[1:], "hDRVvn:i:c:l:u:",
		["help", "noDelete", "noRegister", "noView", "viewLite", "numProducts", "numIReports", "lid", "url"])
except getopt.GetoptError, err:
	print str(err)	  #will print something like "option -a not recognized"
	usage()
for o, a in opts:
	if o in ("-h", "--help"):
		usage()
	elif o in ("-D", "--noDelete"):
		doDelete = False
	elif o in ("-R", "--noRegister"):
		doRegister = False
	elif o in ("-V", "--noView"):
		doView = False
	elif o in ("-v", "--viewLite"):
		viewLite = True
	elif o in ("-n", "--numProducts"):
		numProducts = int(a)
	elif o in ("-i", "--numIntermediateReports"):
		numIReports = int(a)
	elif o in ("-c", "--lidCounter"):
		counterBase = int(a)
	elif o in ("-l", "--lid"):
		lidBase = a
	elif o in ("-u", "--url"):
		urlBase = a
	else:
		usage()
if len(args) != 0: usage()
myX = pds4regURL(urlBase)

#if __name__ == "__main__":
if doRegister:
	doWork(myX.lidRegister, templateRegister, "REGSTR")
if doView:
	doWork(myX.lidView, templateGUID, "VIEW")
if doDelete:
	doWork(myX.lidDelete, templateGUID, "DELETE")

